@@ -1,4 +1,8 @@
-#[derive(Clone, Debug, sqlx::FromRow)]
+#[derive(Clone, Debug)]
+#[cfg_attr(
+    any(feature = "sqlite", feature = "postgres", feature = "mysql",),
+    derive(sqlx::FromRow)
+)]
 pub struct Account {
     pub guid: String,
     pub name: String,
@@ -13,9 +17,11 @@ pub struct Account {
     pub placeholder: Option<i32>,
 }
 
+#[cfg(any(feature = "sqlite", feature = "postgres", feature = "mysql",))]
 impl<'q> Account {
     // test schemas on compile time
     #[allow(dead_code)]
+    #[cfg(feature = "sqlite")]
     fn test_schemas() -> sqlx::query::Map<
         'q,
         sqlx::Sqlite,
@@ -363,6 +369,7 @@ mod tests {
     use super::*;
     use futures::executor::block_on;
 
+    #[cfg(feature = "sqlite")]
     mod sqlite {
         use super::*;
 
@@ -447,6 +454,8 @@ mod tests {
             assert_eq!(3, result.len());
         }
     }
+
+    #[cfg(feature = "postgres")]
     mod postgresql {
         use super::*;
 
@@ -532,6 +541,7 @@ mod tests {
         }
     }
 
+    #[cfg(feature = "mysql")]
     mod mysql {
         use super::*;
 
